#include "../pp.h"
#include <iostream>

// Call pp directly
// instead "okl::pp(...);"
// we can just write "pp(...);"
// I tolerate using namespace if it does not happen within a header file
using namespace okl;


// just a shortcut
static void sleep_ms(unsigned int msDuration)
{
    std::this_thread::sleep_for(std::chrono::milliseconds(msDuration));
}


// will be executed with an other thread
static void doSomething()
{
    unsigned msWait = 100;
    pp(4, "doSomething");
    for (unsigned i = 0; i < 5; i++)
    {
        pp(8, "wait for: $ ms", msWait);
        sleep_ms(msWait);

    }
    pp(4, "~doSomething");
}

// Example structure
struct Point
{
    double x=0;
    double y=0;

    // to make pp work directly with user defined structures we need to overload <<
    friend std::ostream & operator<<(std::ostream & out, const Point & pt)
    {
        out << xprintF("{$,$}", pt.x, pt.y);
        return out;
    }
};


// Example structure
struct Line
{
    Point pt1,pt2;

    // to make pp work directly with user defined structures we need to overload <<
    friend std::ostream & operator<<(std::ostream & out, const Line & line)
    {
        out << xprintF("{$,-,$}", line.pt1, line.pt2);
        return out;
    }
};


static void test_geo_stuff(unsigned int space=0)
{
    pp(space, "test_pp::test_geo_stuff");

    Point pt {300,400};
    Point pt2 {300,1200};

    Line line{pt, pt2};

    pp(space+4, "Point $ is on the Line $ !", pt, line);

    pp(space, "test_pp::~test_geo_stuff");
}

int main()
{
    pp_set_thread_ID_main();

    pp("Hello World's easy logging!");
    pp("some examples...");

    pp("Velocity:$ after $ sec", 9.81*3, 3);

    std::string s = "Hi coderz";
    pp("s:'$'", s);
    pp();

    const double pi = 3.14159265359;
    pp("$", xprf("PI:%2.2f", pi));
    pp(4, "$", xprf("PI:%2.3f", pi));
    pp(8, "$", xprf("PI:%2.4f", pi));

    std::thread th(doSomething);


    sleep_ms(100);
    pp("I am running in main thread");
    th.join();

    pp();

    test_geo_stuff(4);



    pp("End");



    return 0;
}
